% function displayTemplateAndMatching

% Display active basis and adaboost templates;
% and display matched templates on example images.

sx = 120; sy = 120;

% Change the following parameter to control how many features to show for the active basis template. 
nF = 22;
texNF = 25;
% gain_thres = 0.4;

%% active basis template from 5 training positives
iRun = 8;
addpath('sigmoidAdaptive/');
if ~exist(sprintf('sigmoidAdaptive/model_run%d.mat',iRun),'file')
    cd('sigmoidAdaptive/');
    tilt(iRun);
    ablearn(iRun);
    cd('../');
end
load(sprintf('sigmoidAdaptive/model_run%d.mat',iRun));

scale = .7;  % scale of Gabors
norient = 15;  % number of orientations
[allfilter, allsymbol] = makefilter(scale, norient);  % generate Gabor filters 


Mx = Mx(1:nF); My = My(1:nF); Mi = Mi(1:nF); Mm = Mm(1:nF); Mm1 = Mm1(1:nF); Totalsketch = nF;
sym = zeros(sx,sy);
for k = 1:nF
    ori = Mi(k) + 1;
    col = My(k);
    row = Mx(k);
    sym = drawGaborSymbol( sym, allsymbol, row, col, ori, sqrt(Mm(k)) );
end
towrite = -sym;
towrite = uint8(255 * (towrite-min(towrite(:)))/(max(towrite(:))-min(towrite(:))));

imwrite(towrite,sprintf('ActiveBasisTemplate_sketch%d.png',nF));
[y x c] = size(towrite);
figure('Units','Pixels','Resize','off',...
'Position',[100 100 x y],'PaperUnit','points',...
'PaperPosition',[0 0 x y]);
axes('position',[0 0 1 1]);
imshow(towrite, []);
axis off
saveas(gcf, sprintf('ActiveBasisTemplate_sketch%d.eps',nF), 'eps');
basisSym = sym;
basisT = sym;

%% active basis template matching
load(sprintf('RandomSplits/split%d.mat',iRun),'posTrainImages');
addpath('sigmoidAdaptive');
posFolder = 'posAll/';  % folder of positive testing images
posTestImages = posTrainImages;
posTestImagePath = cell(1,length(posTestImages));
for i = 1:length(posTestImages)
    posTestImagePath{i} = [posFolder posTestImages{i}];
end
% posTestImagePath = posTestImagePath(1:19);
nimage = length(posTestImagePath); % # images
Isize = zeros(nimage, 2);
I = cell(1,nimage);
for i = 1 : nimage
    tmpIm = imread(posTestImagePath{i});
    if size(tmpIm,3) == 3
        tmpIm = rgb2gray(tmpIm);
    end
    I{i} = imresize(double(tmpIm),[sx sy]); 
    Isize(i, :) = size(I{i});
end

Sx = zeros(1, nimage) + sx; 
Sy = zeros(1, nimage) + sy; 

% filter the testing images 
disp('start filtering');
tic
allfiltered = applyfilterfftsame(I, allfilter);  % filter testing images
disp(['filtering time: ' num2str(toc) ' seconds']);

% sigmoid transformation
Ctransform(nimage, norient, allfiltered, Sx, Sy, Upperbound);

% prepare the spaces for C code
Asym = cell(1, nimage);   % symbolic plot for each image with active Gabor
for i = 1 : nimage
    Asym{i} = zeros(size(I{1}));  
end
tss = zeros(nimage, Totalsketch);  % total scores for testing images

% pass on the pointers to C for heavy computation
disp('start testing');
tic
bkp = zeros([nimage norient]);
Ctesting(nimage, norient, allfiltered, C, h, sx, sy,...
    Mi, Mx, My, Mm, Mm1, allsymbol(1, :), Asym, Lrange,...
    Orange, sub, Totalsketch, Upperbound, tss, M, lam, e, lz, bkp);

K = nimage;
[tsssort, ind] = sort(tss);
J = cell(1,nimage*2+2);
J{1} = -sym;
J{2} = [];
for i = 1 : K 
    J{2*i+1} = I{ind(nimage-i+1)};
    J{2*i+2} = -(Asym{ind(nimage-i+1)}); 
end
towrite_sketch = displayImages(J(3:end),nimage,sx,sy);


% texture template matching
load(sprintf('oriHistGabor17/model_run%d.mat',iRun));

% compute orientation histograms
hI = cell(size(allfiltered,1),size(allfiltered,2));
for i = 1:size(hI,1)
    for j = 1:size(hI,2)
        hI{i,j} = zeros(size(allfiltered{1,1}));
    end
end

varMap = 1.5*ones(size(allfiltered{1,1}));

sub = max(floor(ax/5),floor(h/4));
addpath('oriHistGabor17');
CGraHist(nimage,norient,allfiltered,hI,varMap,Upperbound,sx,sy,ax,ay,h,sub);

% pass on the pointers to C for heavy computation
disp('start testing');
tic
nimage = size(hI,1);
norient = size(hI,2);

tss = zeros(nimage,maxNF);
texAsym = cell(nimage,1);
for i = 1:nimage
    texAsym{i} = zeros(sx,sy);
    for iF = 1:texNF
        tmpHI = zeros(1,norient);
        for o = 1:norient
            tmpHI(o) = hI{i,o}(Mx(iF),My(iF));
        end
        tss(i,iF) = - sum((tmpHI-selectedMeanHist(iF,:)).^2)/2/(minVar(iF)+1e-10) - log(1e-10+sqrt(2*pi*minVar(iF))) -log(dZ(iF));
        vec = tmpHI;
        vec = vec.^3 + 1e-10;
        vec = vec/sum(vec);
        for ori = 1:norient
            symbol = allsymbol{ori};
            texAsym{i}(Mx(iF)+(-h:h),My(iF)+(-h:h)) = ...
                texAsym{i}(Mx(iF)+(-h:h),My(iF)+(-h:h)) ...
                + vec(ori) * symbol * tss(i,iF) * 5;
        end
    end
end

for i = 1 : nimage
    mixedSym = zeros(sx,sy,3);
    basisSym = -J{2*i+2};
    mixedSym(:,:,2) = mixedSym(:,:,2) - basisT ;
    mixedSym(:,:,1) = mixedSym(:,:,1) - basisT ;
    mixedSym(:,:,3) = mixedSym(:,:,3) - basisT ;
    texSym = texAsym{ind(nimage-i+1)};
    mixedSym(:,:,2) = mixedSym(:,:,2) - 0.5*texSym .* double(texSym>0);
    mixedSym(:,:,3) = mixedSym(:,:,3) - 0.5*texSym .* double(texSym>0);
    mixedSym = 255 * (mixedSym-min(mixedSym(:)))/(max(mixedSym(:))-min(mixedSym(:)));
    J{2*i+2} = mixedSym; 
end

towrite = displayImages(J(3:end),nimage,sx,sy);
imwrite(towrite,'matched.png');
figure;imshow(towrite,[]);

% [y x c] = size(towrite);
% figure('Units','Pixels','Resize','off',...
% 'Position',[100 100 x y],'PaperUnit','points',...
% 'PaperPosition',[0 0 x y]);
% axes('position',[0 0 1 1]);
% imshow(towrite, []);
% axis off
% saveas(gcf, sprintf('matched.eps'), 'eps');

%% texture template

load(sprintf('OriHistGabor17/model_run%d.mat',iRun));

tx = sx; ty = sy;
% Mx = template.Mx; My = template.My; gain = template.gain;

texSym = zeros(sx,sy);
for iF = 1:texNF
    vec = selectedMeanHist(iF,:);
    vec = vec.^3 + 1e-10;
    vec = vec/sum(vec);
    for ori = 1:norient
        symbol = allsymbol{ori};
        texSym(Mx(iF)+(-h:h),My(iF)+(-h:h)) = ...
            texSym(Mx(iF)+(-h:h),My(iF)+(-h:h)) ...
            + vec(ori) * symbol;
    end
end

%% mixed template 
basisSym = imresize(basisT,[tx ty],'bilinear');
texSym = imresize(texSym,[tx ty],'bilinear');
mixedSym = zeros(tx,ty,3);
mixedSym(:,:,2) = mixedSym(:,:,2) - basisSym ;
mixedSym(:,:,1) = mixedSym(:,:,1) - basisSym ;
mixedSym(:,:,3) = mixedSym(:,:,3) - basisSym ;
mixedSym(:,:,2) = mixedSym(:,:,2) - 1.5*texSym .* double(texSym>0);
mixedSym(:,:,3) = mixedSym(:,:,3) - 1.5*texSym .* double(texSym>0);
mixedSym = 255 * (mixedSym-min(mixedSym(:)))/(max(mixedSym(:))-min(mixedSym(:)));
mixedSym = uint8(mixedSym);
% figure;
figure;imshow(mixedSym);
imwrite(mixedSym,'mixedT.png');
