function tss = ABtest_one(iRun,imagePaths,ax)

load(sprintf('model_run%d.mat',iRun)); % load trained shared-sketch model

nimage = length(imagePaths); % # images
Isize = zeros(nimage, 2);
I = cell(1,nimage);
for i = 1 : nimage
    im = imread(imagePaths{i});
    if size(im,3) == 3
        im = rgb2gray(im);
    end
    im = double(im);
    im = imresize(im,[sx sy]);
    I{i} = im;
    Isize(i, :) = size(I{i});
end

Sx = zeros(1, nimage) + sx; 
Sy = zeros(1, nimage) + sy; 

% filter the testing images 
disp('start filtering');
tic
allfiltered = applyfilterfftsame(I, allfilter);  % filter testing images
disp(['filtering time: ' num2str(toc) ' seconds']);

% sigmoid transformation
mex Ctransform.c; 
Ctransform(nimage, norient, allfiltered, Sx, Sy, Upperbound);

% compute local mean
localMeanMap = cell(nimage,norient);

% compute local mean maps using (the integral image) fft 
averageFilter = ones(2*ax+1,2*ax+1)/(2*ax+1)/(2*ax+1);
for i = 1:nimage
    for o = 1:norient
        localMeanMap{i,o} = filter2(averageFilter,allfiltered{i,o},'same');
    end
end

% prepare the spaces for C code
Asym = cell(1, nimage);   % symbolic plot for each image with active Gabor
for i = 1 : nimage
    Asym{i} = zeros(size(I{1}));  
end
tss = zeros(nimage, Totalsketch);  % total scores for testing images: (the scores are not accumulative !) 

% pass on the pointers to C for heavy computation
disp('start testing');
tic
Ctesting(nimage, norient, allfiltered, C, h, sx, sy,...
    Mi, Mx, My, Mm, Mm1, allsymbol(1, :), Asym, Lrange,...
    Orange, sub, Totalsketch, Upperbound, tss, M, lam, e, lz);

% post-process the testing scores
sinsh = zeros(norient, 2*Lrange+1); 
cossh = zeros(norient, 2*Lrange+1); 
for o = 1:norient
    theta = pi*(o-1)/norient;
    for l = -Lrange:Lrange
        sinsh(o,l+Lrange+1) = floor(l*sub*sin(theta)+.5); 
        cossh(o,l+Lrange+1) = floor(l*sub*cos(theta)+.5); 
    end
end

for i = 1:nimage
    for iF = 1:Totalsketch
        [r xm ym om] = shiftmax(allfiltered(i,:),Mx(iF),My(iF),Mi(iF)+1,Lrange,Orange,sinsh,cossh);
        % fit a local texture model
        m = localMeanMap{i,om}(xm,ym);
        [dif ind] = min(abs(e-m));
        ind = max(1,ind); ind = min(ind,length(e));
        texScore = lam(ind) * r - lz(ind);
        tss(i,iF) = tss(i,iF) - texScore;
    end
end

disp(['mex-C testing time: ' num2str(toc) ' seconds']);

%%

function [lm xm ym om] = shiftmax(map,x,y,o,Lrange,Orange,sinsh,cossh)
% o starts from 1
norient = size(map,2);
lm = -1e10;
for l = -Lrange:Lrange
    x1 = x + cossh(o,l+Lrange+1); 
    y1 = y + sinsh(o,l+Lrange+1);   % shifting in normal direction
    for de = -Orange:Orange
        o1 = de + o;
        while o1 > norient
            o1 = o1 - norient;
        end
        while o1 < 1
            o1 = o1 + norient;
        end
        if map{o1}(x1,y1) > lm
            lm = map{o1}(x1,y1);
            xm = x1;
            ym = y1;
            om = o1;
        end
    end
end
